@@ -52,6 +52,8 @@ using std::pair;
 #include "check_list.hpp"
 #include "lsort.hpp"
 
+#include "iostream.hpp"
+
 #include "gettext.h"
 
 typedef unsigned int   u32int;
@@ -158,7 +160,7 @@ static inline const char * get_affix(const char * d) {
 
 static inline const char * get_category(const char * d) {
   int word_size = get_word_size(d);
-  if (get_flags(d) & HAVE_AFFIX_FLAG & HAVE_CATEGORY_FLAG) 
+  if (get_flags(d) & (HAVE_AFFIX_FLAG | HAVE_CATEGORY_FLAG)) 
     return d + strlen(d + word_size +  1) + 1;
   else if (get_flags(d) & HAVE_CATEGORY_FLAG)
     return d + word_size +  1;
@@ -478,6 +480,7 @@ namespace {
     const Jump * jump1;
     const Jump * jump2;
     const char * cur;
+    const char * prev;
     int level;
     bool invisible_soundslike;
 
@@ -492,9 +495,12 @@ namespace {
   WordEntry * ReadOnlyDict::SoundslikeElements::next(int stopped_at) {
 
     //CERR << level << ":" << stopped_at << "  :";
-    //CERR << jump1->data << ":" << jump2->sl << "\n";
+    //CERR << jump1->sl << ":" << jump2->sl << "\n";
+
+  loop:
 
     const char * tmp = cur;
+    const char * p;
 
     if (level == 1 && stopped_at < 2) {
 
@@ -523,7 +529,7 @@ namespace {
 
     } else if (level == 2) {
 
-      cur = tmp = obj->word_block + jump2->loc;
+      tmp = cur = obj->word_block + jump2->loc;
       level = 3;
 
     } else if (get_offset(cur) == 0) {
@@ -543,6 +549,27 @@ namespace {
 
     cur = get_next(cur); // this will be the NEXT item looked at
 
+    p = prev;
+    prev = tmp;
+    if (p) {
+      // PRECOND:
+      // unless stopped_at >= LARGE_NUM
+      //     strlen(p) >= stopped_at
+      //     (stopped_at >= 3) implies 
+      //         strncmp(p, tmp, 3) == 0 if !invisible_soundslike
+      //         strncmp(to_sl(p), to_sl(tmp), 3) == 0 if invisible_soundslike
+      if (stopped_at == 3) {
+        if (p[3] == tmp[3]) goto loop;
+      } else if (stopped_at == 4) {
+        if (p[3] == tmp[3] && tmp[3] &&
+            p[4] == tmp[4]) goto loop;
+      } else if (stopped_at == 5) {
+        if (p[3] == tmp[3] && tmp[3] &&
+            p[4] == tmp[4] && tmp[4] &&
+            p[5] == tmp[5]) goto loop;
+      }
+    }
+    
     data.word = tmp;
     data.word_size = get_word_size(tmp);
     if (invisible_soundslike) {
@@ -550,9 +577,11 @@ namespace {
       data.what = WordEntry::Word;
     } 
     data.intr[0] = (void *)tmp;
+    
     return &data;
 
   jquit:
+    prev = 0;
     if (!*tmp) return 0;
     data.word = tmp;
     data.word_size = !tmp[1] ? 1 : !tmp[2] ? 2 : 3;
@@ -921,7 +950,7 @@ namespace {
           prev->aff = aff;
           aff += l1;
           for (const char * p = cur->aff; *p; ++p) {
-            if (memchr(prev->aff, l1, *p)) continue;
+            if (memchr(prev->aff, *p, l1)) continue;
             *aff = *p;
             ++aff;
           }
